MODULE	TimeDate;
IMPORT	Console, Platform, GPIO := WiringPi, 
	DS1302S := gpioDS1302S, Strings,
	CRC16, PsdRndmSqnc, Args;

CONST
	ansiDgt = ORD (030X);

	AlwaysONE = {7};

	DS1302RgstrRead = AlwaysONE + (( -{DS1302S.nClock3Wire} ) * {DS1302S.nClock3Wire}) + 
			  ((*- {DS1302S.nWr3Wire}) *) {DS1302S.nWr3Wire});
	DS1302RgstrWrite = AlwaysONE + (( -{DS1302S.nClock3Wire} ) * {DS1302S.nClock3Wire}) + 
			  ((- {DS1302S.nWr3Wire}) * {DS1302S.nWr3Wire});
	DS1302RAMRead = AlwaysONE + ((* -{DS1302S.nClock3Wire}  *) {DS1302S.nClock3Wire}) + 
			  ((* -{DS1302S.nWr3Wire}) *) {DS1302S.nWr3Wire});
	DS1302RAMWrite = AlwaysONE + ((* -{DS1302S.nClock3Wire}  *) {DS1302S.nClock3Wire}) + 
			  ((- {DS1302S.nWr3Wire}) * {DS1302S.nWr3Wire});

	DateCmmnd = 'sudo date --u MMDDhhmmYYYY';

	maxScnds = 60;
	maxMnts = 60;
	maxHrs = 24;
	maxMnth = 13;
	
	MSKzone = 3;

TYPE
	pstr = POINTER TO ARRAY OF SHORTCHAR;

VAR
	SysRslt, d, i :INTEGER;
	ad :ARRAY 8 OF INTEGER;
	s :SET;
	strng :ARRAY 13 OF SHORTCHAR;
	NewTmDt :BOOLEAN;
	cmmnd :pstr;
	DayNmbrsTbl :ARRAY 13 OF INTEGER;

(* ######################################################### *)
PROCEDURE	GPIOTo3Wire;
	BEGIN
		SysRslt := Platform.System ("gpio export 13 out");
		SysRslt := Platform.System ("gpio -g mode 13 down");
		SysRslt := Platform.System ("gpio export 6 out");
		SysRslt := Platform.System ("gpio -g mode 6 down");
		SysRslt := Platform.System ("gpio export 5 out");
		SysRslt := Platform.System ("gpio -g mode 5 down");
		
		GPIO.SetupGpio;	
		
		GPIO.DelayMicroseconds (300);
		GPIO.DigitalWrite (DS1302S.nRST, GPIO.LOW);
		GPIO.DigitalWrite (DS1302S.SCK, GPIO.LOW);
		GPIO.DigitalWrite (DS1302S.IO, GPIO.LOW);
		
	END GPIOTo3Wire;

(* ######################################################### *)
PROCEDURE	TmkprGetChckData;
	VAR
		i, d :INTEGER;
	BEGIN
		FOR i := 018H TO 01EH DO
		    d := ORD (DS1302RAMRead + BITS (ASH (i, 1)));
		    DS1302S.BusCycle (d);
		    DS1302S.RAM [(i - 018H)] := d;
		END;
		
	END TmkprGetChckData;

(* ######################################################### *)
PROCEDURE	TmkprChckDataPrpr;
	CONST
		LSB = BITS (00FFH);
		ScndB = BITS (0FF00H);
	VAR
		s :SET;
		d, i :INTEGER;
	BEGIN
		PsdRndmSqnc.Shifts (16);
		d := PsdRndmSqnc.Rgstr ();
		s := BITS (d);
		DS1302S.RAM [0] := ORD (s * LSB);
		DS1302S.RAM [1] := ASH (ORD (s * ScndB), -8);
		
		PsdRndmSqnc.Shifts (16);
		d := PsdRndmSqnc.Rgstr ();
		s := BITS (d);
		DS1302S.RAM [2] := ORD (s * LSB);
		DS1302S.RAM [3] := ASH (ORD (s * ScndB), -8);
		
		PsdRndmSqnc.Shifts (16);
		d := PsdRndmSqnc.Rgstr ();
		s := BITS (d);
		DS1302S.RAM [4] := ORD (s * LSB);			(* Сформированы новые контрольные данные*)
		
		CRC16.Start;
		FOR i := 0 TO 4 DO
		    CRC16.CRC (DS1302S.RAM [i]);
		END;							(* Цикл подсчёта контрольной суммы *)
		
		d := CRC16.Rslt ();
		s := BITS (d);
		DS1302S.RAM [5] := ORD (s * LSB);
		DS1302S.RAM [6] := ASH (ORD (s * ScndB), -8);		(* Новые контрольные данные с контрольной суммой *)
		
	END TmkprChckDataPrpr;


(* ######################################################### *)
PROCEDURE	DayNmbrsTblInit;
	BEGIN
		DayNmbrsTbl [0] := 00; DayNmbrsTbl [1] := 32; DayNmbrsTbl [2] := 30;
		DayNmbrsTbl [3] := 32; DayNmbrsTbl [4] := 31; DayNmbrsTbl [5] := 32;
		DayNmbrsTbl [6] := 31; DayNmbrsTbl [7] := 32; DayNmbrsTbl [8] := 32;
		DayNmbrsTbl [9] := 31; DayNmbrsTbl [10] := 32; DayNmbrsTbl [11] := 31;
		DayNmbrsTbl [12] := 32;
		
	END DayNmbrsTblInit;

(* ######################################################### *)
PROCEDURE	NewTimeDateChck (VAR tmkpr :DS1302S.TimeDate; VAR nwtmdt :BOOLEAN);
	CONST
		NoPrmtrs = -14;
		IllglPrmCount = -12;
		IllglPrfx = -10;
		NonCrrctLEN = -8;
		IllglChars = -6;
		IllglDgts = -4;
		IllglNmbrs = -2;
		None = 0;
	VAR
		utc, b, leaf :BOOLEAN;
		error, n, dy, dd, dh, i :INTEGER;
		DayNmbrsTbl :ARRAY 13 OF INTEGER;

	PROCEDURE	ByteToBCD (d :INTEGER) :INTEGER;
		BEGIN
			RETURN (((d DIV 10) * 16) + (d MOD 10));
		END ByteToBCD;

	BEGIN
		IF ((Args.argc - 1) = 2)				(* Проверка числа параметров в командной строке *)
		    THEN
			error := None;					(* Число параметров корректное *) 
			Args.Get (1, strng);				(* Взят первый *)
		    ELSIF ((Args.argc - 1) = 0)
			   THEN
			       error := NoPrmtrs
			   ELSE
			       error := IllglPrmCount			(* Число параметров не корректное *)
		END;
		
		IF ((error = None) & 
		    ((strng = '--m') OR (strng = '--u')))		(* Проверка корректности первого параметра *)
		    THEN
			IF (strng [2] = 'u')				(* Определим признак часового пояса *)
			    THEN
				utc := TRUE;
			    ELSE
				utc := FALSE
			END;
			Args.Get (2, strng);				(* Взят второй параметр из командной стоки *)
			n := LEN (strng$)				(* Кол-во символов в нём *)
		    ELSIF (error > IllglPrfx)
			   THEN
			       error := IllglPrfx			(* Первый параметр не корректен *)
		END;
		
		IF ((error = None) &
		    (n = LEN ('MMDDhhmmYYYY')))				(* Проверка достаточного кол-ва символов во втором параметре *)
		    THEN
			i := 0;
			WHILE ((error = None) &
			       (i < n)) DO
				IF ((strng [i] < '0') OR 
				    (strng [i] > '9'))
				    THEN				(* Во втором параметре не только цифры*)
					error := IllglChars		(* Новых данных для DS1302S не будет *)
				END;
				i := (i + 1)
			END;						(* Проверка, что второй параметр - только цифры *)
		    ELSIF (error > NonCrrctLEN)
			   THEN
			       error := NonCrrctLEN			(* Второй параметр некорректен *)
		END;
		
		IF ((error > IllglDgts) &
		    (strng [0] < '2') & (strng [2] < '4') &
		    (strng [4] < '3') & (strng [6] < '6') & 
		    (strng [8] = '2') & (strng [9] = '0'))
		    THEN						(* Параметры командной строки синтаксически верны *)
			DayNmbrsTblInit;
			dh := ((ORD (strng [4]) - ansiDgt) * 10)
				+ (ORD (strng[5]) - ansiDgt);		(* bin Час *)
			dd := ((ORD (strng [2]) - ansiDgt) * 10)
				+ (ORD (strng[3]) - ansiDgt);		(* bin Дата *)
			n := ((ORD (strng [0]) - ansiDgt) * 10)
			      + (ORD (strng[1]) - ansiDgt);		(* bin Месяц *)
			dy := ((ORD (strng [10]) - ansiDgt) * 10)
			       + (ORD (strng[11]) - ansiDgt);		(* bin Год *)
			       
			leaf := (((2000 + dy) MOD 4) = 0)
		    ELSIF (error > IllglDgts)
			   THEN
			       error := IllglDgts
		END;
		
		IF ((error > IllglNmbrs) &
		    (n # 0) & (n < maxMnth) &
		    (dh < maxHrs) & (dd # 0) & 
		    (dd < DayNmbrsTbl [n]))
		    THEN
			IF (utc = FALSE)
			    THEN
				dh := dh - MSKzone;			(* bin Час по UTC *)
				
				IF (dh < 0)
				    THEN				(* По UTC ещё предыдущие сутки *)
					dh := dh + maxHrs;
					
					IF (dd # 1)
					    THEN
						dd := dd - 1;
					    ELSE			(* предыдущего месяца *)
						IF (n # 1)
						    THEN
							n := (n - 1)
						    ELSE		(* и года *)
							n := 12;
							dy := (dy - 1)
						END;
						
						dd := (DayNmbrsTbl [n] - 1);
						
						IF ((leaf = FALSE) & 
						    (n = 2))
						    THEN
							dd := (dd - 1)	(* Дополнительная коррекция невисокосного февраля *)
						END
					END
				END
			END;
			
			tmkpr.Scnds := 00H;
			tmkpr.Mnts := ((ORD (strng [6]) - ORD (030X)) * 16)
				       + (ORD (strng[7]) - ORD (030X));
			tmkpr.Hrs := ByteToBCD (dh);
			tmkpr.Dt := ByteToBCD (dd);
			tmkpr.Mnth := ByteToBCD (n);
			tmkpr.Year := ByteToBCD (dy) 
		    ELSIF (error > IllglNmbrs)
			   THEN
			       error := IllglNmbrs
		END;
		
		CASE error OF
		   | NoPrmtrs: 
		   | IllglPrmCount: Console.String ('*** Erroneous parameters count.'); Console.Ln;
		   | IllglPrfx:  Console.String ('*** Improper prefix.'); Console.Ln;
		   | NonCrrctLEN: Console.String ('*** Improper length of string.'); Console.Ln;
		   | IllglChars: Console.String ('*** Digits allowed only.'); Console.Ln;
		   | IllglDgts:  Console.String ('*** Improper digits.'); Console.Ln;
		   | IllglNmbrs: Console.String ('*** Improper numbers.'); Console.Ln;
		   ELSE Console.String ('New date\time will be set.'); Console.Ln;
		END;
		
		IF (error = None)
		    THEN
			nwtmdt := TRUE;
		    ELSE
			nwtmdt := FALSE;
		END;
		
	END NewTimeDateChck;

(* ######################################################### *)
PROCEDURE	DateTimeMnlEnter (VAR td :ARRAY OF SHORTCHAR);
	CONST
		sl = LEN ('--u MMDDhhmmYYYY');
		NonCrrctLEN = -10;
		NonCrrctPrfx = -8;
		IllglChars = -6;
		IllglDgts = -4;
		IllglNmbrs = -2; 
		None = 0;
	
	TYPE
		p = POINTER TO ARRAY OF SHORTCHAR;

	VAR
		utc, leaf :BOOLEAN;
		g, i, n, dh, dm, dd, dy, error :INTEGER;
		m :p;
		DayNmbrsTbl :ARRAY 13 OF INTEGER;
	
	BEGIN
		NEW (m, (sl + 1));
		REPEAT
		      error := None;
		      Console.ReadLine (m^); Console.Flush ();
		      
		      IF (LEN (m^$) # sl)				(* Проверка достаточного размера строки данных *)
			  THEN
			      error := NonCrrctLEN;
		      END;
		      
		      IF ((error > NonCrrctPrfx) & 
		          ~((Strings.Pos ('--u', m^, 0) = 0) OR 
			   (Strings.Pos ('--m', m^, 0) = 0)))		(* Наличие допустимого префикса *)
			  THEN
			      error := NonCrrctPrfx;
		      END;
		      
		      IF (error = None)
			  THEN
			      IF (m^ [2] = 'u')				(* Выставлен признак часового пояса *)
			          THEN
				      utc := TRUE;
				  ELSE
				      utc := FALSE
			      END;
			      
			      g := Strings.Pos ('MM', '--u MMDDhhmmYYYY', 0);
			      i := g;
			      n := (Strings.Pos ('YYYY', '--u MMDDhhmmYYYY', 0) + 4);
		      END;
		     
		      WHILE ((error = None) &
			     (i < n)) DO
			     IF ((m^ [i] < '0') OR
				 (m^ [i] > '9'))
				 THEN
				     error := IllglChars
			     END;
			     i := i + 1;
		      END;						(* Цикл проверки - только цифры *)
		     
		      IF ((error > IllglDgts) &
		          (m^ [g] < '2') & (m^ [(g + 2)] < '4') &	(* Месяц начинается до "2", дата начинается до "4" *)
			  (m^ [(g + 4)] < '3') & (m^ [(g + 6)] < '6') &	(* Час начинается до "3", минуты начинаются до "6" *)
			  (m^ [(g + 8)] = '2') & (m^ [(g + 9)] = '0'))	(* Ограничимся 21-м веком *)
			  THEN
			      DayNmbrsTblInit;
			      dh := ((ORD (m^ [(g + 4)]) - ansiDgt) * 10)
			 	    + (ORD (m^ [(g + 5)]) - ansiDgt);	(* bin Час *)
			      dd := ((ORD (m^ [(g + 2)]) - ansiDgt) * 10)
			 	    + (ORD (m^ [(g + 3)]) - ansiDgt);	(* bin Дата *)
			      dm := ((ORD (m^ [g]) - ansiDgt) * 10)
			             + (ORD (m^ [(g + 1)]) - ansiDgt);	(* bin Месяц *)
			      dy := ((ORD (m^ [(g + 10)]) - ansiDgt) * 10)
				     + (ORD (m^ [(g + 11)]) - ansiDgt);	(* bin Год *)
			      leaf := (((2000 + dy) MOD 4) = 0)
			  ELSIF (error > IllglDgts)
				 THEN
				     error := IllglDgts
		      END;
			
		      IF ((error > IllglNmbrs) &
			  (dm # 0) & (dm < maxMnth) &
			  (dh < maxHrs) & (dd # 0) & 
			  (dd < DayNmbrsTbl [dm]))
			  THEN
			      IF (utc = FALSE)
				  THEN
				      dh := dh - MSKzone;		(* bin Час по UTC *)
				     
				      IF (dh < 0)
					  THEN				(* По UTC ещё предыдущие сутки *)
					      dh := dh + maxHrs;
					     
					      IF (dd # 1)
						  THEN
						      dd := (dd - 1);
						  ELSE			(* предыдущего месяца *)
						      IF (dm # 1)
							  THEN
							      dm := (dm - 1)
							  ELSE		(* и года *)
							      dm := 12;
							      dy := (dy - 1);
							     
							      m^ [(g + 10)] := CHR ((dy DIV 10) + ansiDgt);
							      m^ [(g + 11)] := CHR ((dy MOD 10) + ansiDgt)
						      END;
						      m^ [g] := CHR ((dm DIV 10) + ansiDgt);
						      m^ [(g + 1)] := CHR ((dm MOD 10) + ansiDgt);
						     
						      dd := (DayNmbrsTbl [dm] - 1);
						      IF ((leaf = FALSE) & 
							  (dm = 2))
							  THEN
							      dd := (dd - 1)	(* Дополнительная коррекция невисокосного февраля *)
						      END;
					      END;
					      
					      m^ [(g + 2)] := CHR ((dd DIV 10) + ansiDgt);
					      m^ [(g + 3)] := CHR ((dd MOD 10) + ansiDgt)
				      END;
				      
				      m^ [(g + 4)] := CHR ((dh DIV 10) + ansiDgt);
				      m^ [(g + 5)] := CHR ((dh MOD 10) + ansiDgt);
				      m^ [(g - 2)] := 'u'
			      END
			  ELSIF (error > IllglNmbrs)
				 THEN
				     error := IllglNmbrs
		      END;
		      
		      CASE error OF
			 | NonCrrctLEN : Console.String ('*** Improper length of string.');
			 | NonCrrctPrfx : Console.String ('*** Improper prefix.');
			 | IllglChars : Console.String ('*** Digits allowed only.');
			 | IllglDgts :Console.String ('*** Improper digits.');
			 | IllglNmbrs :Console.String ('*** Improper numbers.');
			 ELSE
		      END;
		      Console.Ln;
		      
		      IF (error # None)
			  THEN
			      Console.String ('Reenter right data: ')
		      END;
		    
		UNTIL (error = None);
		
		n := Strings.Pos ('--u MMDDhhmmYYYY', DateCmmnd, 0);
		Strings.Replace (m^, n, td);
		
		m := NIL
		
	END DateTimeMnlEnter;

BEGIN
	(* Получить число параметров командной строки
		не 2 - просто установка системного времени из DS1302S
		2 - проверить префикс: "--m"\"--u" и при правильном префиксе
		    после проверки корректности времени\даты "MMDDhhmmYYYY"
		    (МСК преобразовать в UTC) записать её в DS1302S;
		    установить системное время
	*)
	NewTimeDateChck (DS1302S.Timekeeper, NewTmDt);

	CRC16.CRC16TblInit;
	
	GPIOTo3Wire;							(* Настройка портов ввода-вывода *)

	TmkprGetChckData;						(* Вычитать контрольные данные с CRC (5+2 байта; @ 018H..01EH) из DS1302S. *)

	(*	Посчитать CRC: "0" => дата/время сохранно;
		не "0" => DS1302S как минимум терял питание: выдать предупреждение и требование ввода даты/времени *)
	CRC16.Start;
	FOR i := 0 TO 6 DO
	    CRC16.CRC (DS1302S.RAM [i])
	END;								(* Цикл подсчёта контрольной суммы*)

	NEW (cmmnd, (LEN (DateCmmnd) + 1));
	Strings.Insert (DateCmmnd, 0, cmmnd^);				(* Болванка системной команды установки даты\времени *)
	
	IF (CRC16.Rslt () = 0)
	    THEN							(* Дата\время хранятся в исправности *)
		d := ORD (DS1302RgstrRead + DS1302S.ScndsRgstr);			
		DS1302S.BusCycle (d);					(* Состояние остановки часов *)
			
		s := BITS (d);
		IF (DS1302S.ClckHalt IN s)
		    THEN						(* Запуск стоящих часов *)
			s := s - {DS1302S.ClckHalt};
			d := ORD (BITS (ASH (ORD (s), 8))
			      + DS1302RgstrWrite + DS1302S.ScndsRgstr);
			DS1302S.BusCycle (d);
		END;
		
		Console.String ("Timekeeper is right."); Console.Ln;
		
		IF (NewTmDt = TRUE)
		    THEN
			d := (ASH (DS1302S.Timekeeper.Scnds, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.ScndsRgstr));
			DS1302S.BusCycle (d);
			d := (ASH (DS1302S.Timekeeper.Mnts, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.MntsRgstr));
			DS1302S.BusCycle (d);
			d := (ASH (DS1302S.Timekeeper.Hrs, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.HrsRgstr));
			DS1302S.BusCycle (d);
			d := (ASH (DS1302S.Timekeeper.Dt, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.DateRgstr));
			DS1302S.BusCycle (d);
			d := (ASH (DS1302S.Timekeeper.Mnth, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.MnthRgstr));
			DS1302S.BusCycle (d);
			d := (ASH (DS1302S.Timekeeper.Year, 8)
			      + ORD (DS1302RgstrWrite + DS1302S.YearRgstr));
			DS1302S.BusCycle (d);				(* Новая дата\время сохранены в DS1320S *)
			
			d := (ASH (DS1302S.RAM [4], 8) + DS1302S.RAM [3]);
			PsdRndmSqnc.Set (d);
			TmkprChckDataPrpr;
			
			FOR i := 018H TO 01EH DO
			    d := ORD (BITS (ASH (DS1302S.RAM [(i - 018H)], 8))
				  + DS1302RAMWrite + BITS (ASH (i, 1)));
			    DS1302S.BusCycle (d)
			END;						(* Цикл записи контрольных данных в DS1302S *)
			
			TmkprGetChckData;
			CRC16.Start;
			FOR i := 0 TO 6 DO
			    CRC16.CRC (DS1302S.RAM [i])
			END;						(* Цикл подсчёта контрольной суммы*)
			
			IF (CRC16.Rslt () # 0)
			    THEN
				Console.String ('Attention! Check data is incorrect!');
			    ELSE
				Console.String ("New date\time stored o'k.")
			END;
			Console.Ln;
		    ELSE						(* DS1302S исправен, данных в командной строке нет *)
			d := ORD (DS1302RgstrRead + DS1302S.MntsRgstr);			
			DS1302S.BusCycle (d);				(* Вычитаны минуты *)
			DS1302S.Timekeeper.Mnts := ORD (BITS (d) * DS1302S.MntsMask);
			
			d := ORD (DS1302RgstrRead + DS1302S.HrsRgstr);
			DS1302S.BusCycle (d);				(* Вычитан час *)
			DS1302S.Timekeeper.Hrs := ORD (BITS (d) * DS1302S.HrsMask);
			
			d := ORD (DS1302RgstrRead + DS1302S.DateRgstr);
			DS1302S.BusCycle (d);				(* Вычитана дата *)
			DS1302S.Timekeeper.Dt := ORD (BITS (d) * DS1302S.DateMask);
			
			d := ORD (DS1302RgstrRead + DS1302S.MnthRgstr);
			DS1302S.BusCycle (d);				(* Вычитан месяц *)
			DS1302S.Timekeeper.Mnth := ORD (BITS (d) * DS1302S.MntsMask);
			
			d := ORD (DS1302RgstrRead + DS1302S.YearRgstr);
			DS1302S.BusCycle (d);				(* Вычитан год *)
			DS1302S.Timekeeper.Year := d
		END;
		
		d := Strings.Pos ("mm", cmmnd^, 0);
		cmmnd^[d] := CHR (ASH (DS1302S.Timekeeper.Mnts, -4) + ansiDgt);
		cmmnd^ [(d + 1)] :=  CHR ((DS1302S.Timekeeper.Mnts MOD 16) + ansiDgt);
		
		d := Strings.Pos ('hh', cmmnd^, 0);
		cmmnd^ [d] := CHR (ASH (DS1302S.Timekeeper.Hrs, -4) + ansiDgt);
		cmmnd^ [(d + 1)] := CHR ((DS1302S.Timekeeper.Hrs MOD 16) + ansiDgt);
		
		d := Strings.Pos ('DD', cmmnd^, 0);
		cmmnd^ [d] := CHR (ASH (DS1302S.Timekeeper.Dt, -4) + ansiDgt);
		cmmnd^ [(d + 1)] := CHR ((DS1302S.Timekeeper.Dt MOD 16) + ansiDgt);
		
		d := Strings.Pos ('MM', cmmnd^, 0);
		cmmnd^ [d] := CHR (ASH (DS1302S.Timekeeper.Mnth, -4) + ansiDgt);
		cmmnd^ [(d + 1)] := CHR ((DS1302S.Timekeeper.Mnth MOD 16) + ansiDgt);
		
		d := Strings.Pos ('YYYY', cmmnd^, 0);
		cmmnd^ [d] := '2';
		cmmnd^ [(d + 1)] := '0'; 
		cmmnd^ [(d + 2)] := CHR (ASH (DS1302S.Timekeeper.Year, -4) + ansiDgt);
		cmmnd^ [(d + 3)] := CHR ((DS1302S.Timekeeper.Year MOD 16) + ansiDgt);
	    ELSE							(* Здесь - DS1302S, как минимум, терял питание *)
		PsdRndmSqnc.Init;					(* Проинициирован регистр ПСП *)
		TmkprChckDataPrpr;
		
		FOR i := 018H TO 01EH DO
		    d := ORD (BITS (ASH (DS1302S.RAM [(i - 018H)], 8)) + DS1302RAMWrite + BITS (ASH (i, 1)));
		    DS1302S.BusCycle (d);
		END;							(* Цикл записи контрольных данных в DS1302S *)
		
		TmkprGetChckData;
		
		CRC16.Start;
		FOR i := 0 TO 6 DO
		    CRC16.CRC (DS1302S.RAM [i])
		END;
			
		IF CRC16.Rslt () = 0
		    THEN
			Console.String ('Timekeeper was lost supply but is right.')
		    ELSE
			Console.String ('Timekeeper is wrong or absent.')
		END;
		Console.Ln;
		
		Console.String ('Please, enter actual date\time, "[--u | --m] MMDDhhmmYYYY": ');
		DateTimeMnlEnter (cmmnd^)
	END;
		
(*	Console.String (cmmnd^); Console.String (' --> '); *)
	SysRslt := Platform.System (cmmnd^);
	Console.Ln;

	SysRslt := Platform.System ("date");
	SysRslt := Platform.System ("gpio unexportall");

	cmmnd := NIL;
	
END TimeDate.

DEFINITION PsdRndmSqnc;

  PROCEDURE Init;
  PROCEDURE Rgstr (): INTEGER;
  PROCEDURE Set (d: INTEGER);
  PROCEDURE Shifts (n: INTEGER);

END PsdRndmSqnc.

DEFINITION Strings;

  PROCEDURE Append (IN extra: ARRAY OF SHORTCHAR; VAR dest: ARRAY OF SHORTCHAR);
  PROCEDURE Cap (VAR s: ARRAY OF SHORTCHAR);
  PROCEDURE Delete (VAR s: ARRAY OF SHORTCHAR; pos: INTEGER; n: INTEGER);
  PROCEDURE Extract (IN source: ARRAY OF SHORTCHAR; pos: INTEGER; n: INTEGER; VAR dest: ARRAY OF SHORTCHAR);
  PROCEDURE Insert (IN source: ARRAY OF SHORTCHAR; pos: INTEGER; VAR dest: ARRAY OF SHORTCHAR);
  PROCEDURE Length (IN s: ARRAY OF SHORTCHAR): INTEGER;
  PROCEDURE Match (IN string: ARRAY OF SHORTCHAR; IN pattern: ARRAY OF SHORTCHAR): BOOLEAN;
  PROCEDURE Pos (IN pattern: ARRAY OF SHORTCHAR; IN s: ARRAY OF SHORTCHAR; pos: INTEGER): INTEGER;
  PROCEDURE Replace (IN source: ARRAY OF SHORTCHAR; pos: INTEGER; VAR dest: ARRAY OF SHORTCHAR);

END Strings.

DEFINITION gpioDS1302S;

  CONST
    A03Wire = 1;
    A13Wire = 2;
    A23Wire = 3;
    A33Wire = 4;
    A43Wire = 5;
    Addrss3WireMask = {1, 2, 3, 4, 5};
    ChrgrRgstr = {4};
    ClckHalt = 7;
    CntrlRgstr = {1, 2, 3};
    DS1302BurstRgstrs = 62;
    DS1303WP = 8;
    DateMask = {0, 1, 2, 3, 4, 5};
    DateRgstr = {1, 2};
    Ds0 = {2};
    Ds1 = {3};
    DsOff = {2, 3};
    HrsMask = {0, 1, 2, 3, 4, 5};
    HrsRgstr = {2};
    IO = 6;
    MnthMask = {0, 1, 2, 3, 4};
    MnthRgstr = {3};
    MntsMask = {0, 1, 2, 3, 4, 5, 6};
    MntsRgstr = {1};
    Rs2kOhm = {0};
    Rs4kOhm = {1};
    Rs8kOhm = 1;
    RsOff = {};
    SCK = 5;
    ScndsMask = {0, 1, 2, 3, 4, 5, 6};
    ScndsRgstr = {};
    TCSOff = {};
    TCSOn = {5, 7};
    WeekDayMask = {0, 1, 2};
    WeekDayRgstr = {1, 3};
    YearRgstr = {2, 3};
    n24H = 7;
    nAM = 5;
    nClock3Wire = 6;
    nRST = 13;
    nWr3Wire = 0;

  TYPE
    TimeDate = RECORD
      Scnds: INTEGER;
      Mnts: INTEGER;
      Hrs: INTEGER;
      Dt: INTEGER;
      Mnth: INTEGER;
      Year: INTEGER;
    END;

  VAR
    RAM: ARRAY 31 OF INTEGER;
    Timekeeper: TimeDate;

  PROCEDURE BusCycle (VAR d: INTEGER);

END gpioDS1302S.
