MODULE	gpioDS1302S;

IMPORT	Platform, GPIO := WiringPi;

CONST
	nRST* = 13;							(* P1: 33 *)
	IO* = 6;							(* P1: 31 *)
	SCK* = 5;							(* P1: 29 *)

(* ######################################################### *)
	nWr3Wire* = 0;
	A03Wire* = 1;
	A13Wire* = 2;
	A23Wire* = 3;
	A33Wire* = 4;
	A43Wire* = 5;
	Addrss3WireMask* = {A03Wire..A43Wire};
	nClock3Wire* = 6;
	
(* ######################################################### *)
	ScndsRgstr* = {};
	MntsRgstr* = {A03Wire};
	HrsRgstr* =  {A13Wire};
	DateRgstr* = {A03Wire, A13Wire};
	MnthRgstr* = {A23Wire};
	WeekDayRgstr* = {A03Wire, A23Wire};
	YearRgstr* = {A13Wire, A23Wire};
	CntrlRgstr* = {A03Wire .. A23Wire};
	ChrgrRgstr* = {A33Wire};

	DS1302BurstRgstrs* = 62;


(* ######################################################### *)
	ScndsMask* = {0..6};
	ClckHalt* = 7;

(* ######################################################### *)
	MntsMask* = {0..6};

(* ######################################################### *)
	HrsMask* = {0..5};
	nAM* = 5;
	n24H* = 7;

(* ######################################################### *)
	DateMask* = {0..5};

(* ######################################################### *)
	MnthMask* = {0..4};

(* ######################################################### *)
	WeekDayMask* = {0..2};

(* ######################################################### *)
	DS1303WP* = 8;

(* ######################################################### *)
	Rs0 = 0;
	Rs1 = 1;
	RsOff* = {};
	Rs2kOhm* = {Rs0};
	Rs4kOhm* = {Rs1};
	Rs8kOhm* = Rs0 + Rs1;

	Ds0* = {2};
	Ds1* = {3};
	DsOff* = Ds0 + Ds1;

	TCSOff* = {};
	TCSOn* = {5,7};

(* ######################################################### *)

TYPE
	TimeDate* = RECORD
			  Scnds*, Mnts*, Hrs*,
			  Dt*, Mnth*, Year* :INTEGER;			(* BCD *)
		    END;

VAR
	Timekeeper* :TimeDate;
	RAM* :ARRAY 31 OF INTEGER;

PROCEDURE	BusCycle* (VAR d :INTEGER);
	VAR
		s :SET;
		i, Nbits, dd, sysrslt :INTEGER;
		ReadCycle :BOOLEAN;

	BEGIN
		GPIO.DelayMicroseconds (8);
		GPIO.DigitalWrite (IO, GPIO.LOW);
		GPIO.PinMode (IO, GPIO.OUTPUT);				(* 3Wire_IO is output *)
		GPIO.DelayMicroseconds (20);

		s := BITS (d);

		GPIO.DigitalWrite (nRST, GPIO.HIGH);			(* nRST "0"-->"1" *)
		GPIO.DelayMicroseconds (8);

		IF (nWr3Wire IN s)					(* Direction of transaction check *)
		    THEN
			Nbits := 8;
			ReadCycle := TRUE;
		    ELSE
			Nbits := 16;
			ReadCycle := FALSE
		END;

		FOR i := 0 TO (Nbits - 1) DO				(* Cycle throw all bits of control byte and may be throw all bits data byte *)
		    GPIO.DigitalWrite (SCK, GPIO.LOW);			(* 3WireSCK "1"-->"0"*)
		    IF (i IN s)
			THEN
			    dd := GPIO.HIGH;
			ELSE
			    dd := GPIO.LOW
		    END;
		    GPIO.DigitalWrite (IO, dd);				(* 3WireIO actual data *)
		    GPIO.DelayMicroseconds (5);
		    GPIO.DigitalWrite (SCK, GPIO.HIGH);			(* 3WireSCK "0"-->"1"*)
		    GPIO.DelayMicroseconds (5);
		END;

		IF ReadCycle
		    THEN
			s := {};
			GPIO.PinMode (IO, GPIO.INPUT);			(* 3Wire_IO is input *)
			GPIO.DelayMicroseconds (20);
			FOR i := 0 TO (Nbits - 1) DO
			    GPIO.DigitalWrite (SCK, GPIO.LOW);		(* 3WireSCK "1"-->"0" *)
			    GPIO.DelayMicroseconds (10);
			    dd := GPIO.DigitalRead (IO);
			    IF dd = GPIO.LOW
				THEN
				    s := s - {i};
				ELSE
				    s := s + {i}
			    END;
			    GPIO.DigitalWrite (SCK, GPIO.HIGH);		(* 3WireSCK "0"-->"1" *)
			    GPIO.DelayMicroseconds (5);
			END;
			d := ORD (s * BITS (0FFH));
		END;

		GPIO.DigitalWrite (SCK, GPIO.LOW); 			(* 3WireSCK "1"-->"0" *)
		GPIO.DelayMicroseconds (5);
		GPIO.DigitalWrite (nRST, GPIO.LOW); 			(* nRST "1"-->"0" *)

	END BusCycle;

END gpioDS1302S.
